// Declare margins, width and height
var m = {top: 40, right: 40, bottom: 50, left: 40};
var w = document.getElementById("chart").clientWidth - m.left - m.right;
var h = document.getElementById("chart").clientHeight - m.top - m.bottom;

// Set x-axis to scale
var x = d3.scaleLinear()
    .domain([0, data.length])
    .range([0, w]);

// Set y-axis to scale
var y0 = d3.scaleLinear()
    .domain([0, 100.0])
    .range([h, 0]);
var y1 = d3.scaleLinear()
    .domain([0, 100.00])
    .range([h, 0]);

// Create first line: time to x-axis and level score to y-axis
var line1 = d3.line()
    .x(function(d) { return x(d.time); })
    .y(function(d) { return y0(d.level); })

// Create second line: time to x-axis and cycle to y-axis
var line2 = d3.line()
    .x(function(d) { return x(d.time); })
    .y(function(d) { return y1(d.cycle); })

// Create and position svg chart
var svg = d3.select(".battery-chart").append("svg")
    .attr("width", w + m.left + m.right)
    .attr("height", h + m.top + m.bottom)
    .attr("class", "svg")
    .append("g")
    .attr("transform", "translate(" + m.left + "," + m.top + ")");

// Format data
data.forEach(function(d) {
    d.time = +d.time;
    d.level = +d.level;
    d.cycle = +d.cycle;
});

// Scale x-axis data
x.domain(d3.extent(data, function(d) { return d.time }));

// TODO: Determine how we want to scale the y-axis. Do we always want to
// display the full domain which makes it easier to compare how much a
// battery discharged for each test?  Or do we want to display a truncated
// domain which makes it easier to see changes in the discharge rate for
// partial discharge tests?

// Scale y-axis data against level only. Both lines are using percent
// y0.domain([
//   d3.min(data, function(d) { return Math.max(d.level); }),
//   d3.max(data, function(d) { return Math.max(d.level); })]);

function pathTween(p) {
    var interpolate = d3.scaleLinear()
        .domain([0,1])
        .range(d3.range(1, p.length + 1));
}

// Draw x axis
var gx = svg.append("g")
    .attr("class", "axis axis--x")
    .attr("transform", "translate(0," + h + ")")
    .call(d3.axisBottom(x));

// Draw left y axis
var gy0 = svg.append("g")
    .attr("class", "axis axis--y")
    .call(d3.axisLeft(y0));

dataPerPixel = data.length/w;
dataResampled = data.filter(
    function(d, i) { return i % Math.ceil(dataPerPixel) == 0; }
);

// Draw line 1
svg.append("path")
    .data(dataResampled)
    .attr("class", "line line1")
    .attr("d", line1)
    .transition()
    .duration(2000)
    .attrTween("d", pathTween(line1));

// Draw line 2
svg.append("path")
    .data(dataResampled)
    .attr("class", "line line2")
    .style("stroke", "#CCCCCC")
    .attr("d", line2)
    .transition()
    .duration(2000)
    .attrTween("d", pathTween(line2));

// When resizing the window, redraw chart
function resize() {

    w = document.getElementById("chart").clientWidth - m.left - m.right;
    h = document.getElementById("chart").clientHeight - m.top - m.bottom;

    x.range([0, w]).nice();
    y0.range([h, 0]).nice();
    y1.range([h, 0]).nice();

    x.ticks(Math.max(w/50, 2));
    y0.ticks(Math.max(h/50, 2));
    y1.ticks(Math.max(h/50, 2));

    d3.select(".battery-chart svg")
    .attr("width", w + m.left + m.right)

    svg
    .attr("width", w + m.left + m.right)
    .attr("height", h + m.top + m.bottom)

    /* TODO: Determine width of svg and change ammount of ticks on x-axis
    var tickValue;

    if (w >= 0 && w <= 400) {
    tickValue = 4;
    } else if (w >= 401 && w <= 699) {
    tickValue = 6;
    } else if (w >= 700 && w <= 800) {
    tickValue = 10;
    } else {
    tickValue = 12;
    } */

    gx
    .attr("transform", "translate(0," + h + ")")
    .call(d3.axisBottom(x)
        .ticks(2)
        .tickFormat(function (ts) {
        dec = function (d) { return d3.format("02d")(Math.floor(d)); };
        return dec ( ts /60/60%60) + ":"+ dec( ts /60%60) + ":"+ dec(ts%60);}));

    gy0
    .attr("width", w + m.left + m.right)
    .call(d3.axisLeft(y0)
        .tickFormat(function (d) { return d + '%'; }));

    gy0.selectAll("g")
    .classed("minor", true)
    .select("line")
    .attr("x2", w);

    gy0.selectAll("text")
    .attr("x", 0)
    .attr("dy", -6);

    dataPerPixel = data.length/w;
    dataResampled = data.filter(
    function(d, i) { return i % Math.ceil(dataPerPixel) == 0; }
    );

    svg.selectAll(".line1")
    .datum(dataResampled)
    .attr("d", line1);

    svg.selectAll(".line2")
    .datum(dataResampled)
    .attr("d", line2);
}

d3.select(window).on("resize", resize);

resize();